@@ -2,7 +2,7 @@
  * info.c:  return system-generated metadata about paths or URLs.
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -22,11 +22,13 @@
 
 #include "client.h"
 #include "svn_client.h"
-#include "svn_wc.h"
 #include "svn_pools.h"
 #include "svn_path.h"
+#include "svn_hash.h"
+#include "svn_wc.h"
 
 #include "svn_private_config.h"
+#include "private/svn_wc_private.h"
 
 
 /* Helper: build an svn_info_t *INFO struct from svn_dirent_t DIRENT
@@ -51,8 +53,11 @@ build_info_from_dirent(svn_info_t **info,
   tmpinfo->repos_root_URL       = repos_root;
   tmpinfo->last_changed_rev     = dirent->created_rev;
   tmpinfo->last_changed_date    = dirent->time;
-  tmpinfo->last_changed_author  = dirent->last_author;;
+  tmpinfo->last_changed_author  = dirent->last_author;
   tmpinfo->lock                 = lock;
+  tmpinfo->depth                = svn_depth_unknown;
+  tmpinfo->working_size         = SVN_INFO_SIZE_UNKNOWN;
+  tmpinfo->size                 = dirent->size;
 
   *info = tmpinfo;
   return SVN_NO_ERROR;
@@ -80,6 +85,7 @@ build_info_from_entry(svn_info_t **info,
   /* entry-specific stuff */
   tmpinfo->has_wc_info          = TRUE;
   tmpinfo->schedule             = entry->schedule;
+  tmpinfo->depth                = entry->depth;
   tmpinfo->copyfrom_url         = entry->copyfrom_url;
   tmpinfo->copyfrom_rev         = entry->copyfrom_rev;
   tmpinfo->text_time            = entry->text_time;
@@ -89,6 +95,9 @@ build_info_from_entry(svn_info_t **info,
   tmpinfo->conflict_new         = entry->conflict_new;
   tmpinfo->conflict_wrk         = entry->conflict_wrk;
   tmpinfo->prejfile             = entry->prejfile;
+  tmpinfo->changelist           = entry->changelist;
+  tmpinfo->working_size         = entry->working_size;
+  tmpinfo->size                 = SVN_INFO_SIZE_UNKNOWN;
 
   /* lock stuff */
   if (entry->lock_token)  /* the token is the critical bit. */
@@ -114,7 +123,14 @@ build_info_from_entry(svn_info_t **info,
 
 
 /* Helper func for recursively fetching svn_dirent_t's from a remote
-   directory and pushing them at an info-receiver callback. */
+   directory and pushing them at an info-receiver callback.
+
+   DEPTH is the depth starting at DIR, even though RECEIVER is never
+   invoked on DIR: if DEPTH is svn_depth_immediates, then invoke
+   RECEIVER on all children of DIR, but none of their children; if
+   svn_depth_files, then invoke RECEIVER on file children of DIR but
+   not on subdirectories; if svn_depth_infinity, recurse fully.
+*/
 static svn_error_t *
 push_dir_info(svn_ra_session_t *ra_session,
               const char *session_URL,
@@ -124,6 +140,7 @@ push_dir_info(svn_ra_session_t *ra_session,
               const char *repos_root,
               svn_info_receiver_t receiver,
               void *receiver_baton,
+              svn_depth_t depth,
               svn_client_ctx_t *ctx,
               apr_hash_t *locks,
               apr_pool_t *pool)
@@ -154,7 +171,7 @@ push_dir_info(svn_ra_session_t *ra_session,
 
       path = svn_path_join(dir, key, subpool);
       URL  = svn_path_url_add_component(session_URL, key, subpool);
-     
+
       fs_path = svn_path_is_child(repos_root, URL, subpool);
       fs_path = apr_pstrcat(subpool, "/", fs_path, NULL);
       fs_path = svn_path_uri_decode(fs_path, subpool);
@@ -164,13 +181,19 @@ push_dir_info(svn_ra_session_t *ra_session,
       SVN_ERR(build_info_from_dirent(&info, the_ent, lock, URL, rev,
                                      repos_UUID, repos_root, subpool));
 
-      SVN_ERR(receiver(receiver_baton, path, info, subpool));
+      if (depth >= svn_depth_immediates
+          || (depth == svn_depth_files && the_ent->kind == svn_node_file))
+        {
+          SVN_ERR(receiver(receiver_baton, path, info, subpool));
+        }
 
-      if (the_ent->kind == svn_node_dir)
-        SVN_ERR(push_dir_info(ra_session, URL, path,
-                              rev, repos_UUID, repos_root,
-                              receiver, receiver_baton,
-                              ctx, locks, subpool));
+      if (depth == svn_depth_infinity && the_ent->kind == svn_node_dir)
+        {
+          SVN_ERR(push_dir_info(ra_session, URL, path,
+                                rev, repos_UUID, repos_root,
+                                receiver, receiver_baton,
+                                depth, ctx, locks, subpool));
+        }
     }
 
   svn_pool_destroy(subpool);
@@ -183,8 +206,9 @@ push_dir_info(svn_ra_session_t *ra_session,
 /* Callback and baton for crawl_entries() walk over entries files. */
 struct found_entry_baton
 {
+  apr_hash_t *changelist_hash;
   svn_info_receiver_t receiver;
-  void *receiver_baton;         
+  void *receiver_baton;
 };
 
 static svn_error_t *
@@ -194,26 +218,30 @@ info_found_entry_callback(const char *path,
                           apr_pool_t *pool)
 {
   struct found_entry_baton *fe_baton = walk_baton;
-  svn_info_t *info;
 
   /* We're going to receive dirents twice;  we want to ignore the
      first one (where it's a child of a parent dir), and only print
      the second one (where we're looking at THIS_DIR.)  */
-  if ((entry->kind == svn_node_dir) 
+  if ((entry->kind == svn_node_dir)
       && (strcmp(entry->name, SVN_WC_ENTRY_THIS_DIR)))
     return SVN_NO_ERROR;
 
-  SVN_ERR(build_info_from_entry(&info, entry, pool));
- 
-  return fe_baton->receiver(fe_baton->receiver_baton, path, info, pool);
+  if (SVN_WC__CL_MATCH(fe_baton->changelist_hash, entry))
+    {
+      svn_info_t *info;
+      SVN_ERR(build_info_from_entry(&info, entry, pool));
+      SVN_ERR(fe_baton->receiver(fe_baton->receiver_baton, path, info, pool));
+    }
+  return SVN_NO_ERROR;
 }
 
 
 
-static const svn_wc_entry_callbacks_t 
+static const svn_wc_entry_callbacks2_t
 entry_walk_callbacks =
   {
-    info_found_entry_callback
+    info_found_entry_callback,
+    svn_client__default_walker_error_handler
   };
 
 
@@ -222,45 +250,43 @@ entry_walk_callbacks =
 static svn_error_t *
 crawl_entries(const char *wcpath,
               svn_info_receiver_t receiver,
-              void *receiver_baton,               
-              svn_boolean_t recurse,
+              void *receiver_baton,
+              svn_depth_t depth,
+              apr_hash_t *changelist_hash,
               svn_client_ctx_t *ctx,
               apr_pool_t *pool)
 {
   svn_wc_adm_access_t *adm_access;
   const svn_wc_entry_t *entry;
-  svn_info_t *info;
-  struct found_entry_baton fe_baton;
-  
-  SVN_ERR(svn_wc_adm_probe_open3(&adm_access, NULL, wcpath, FALSE,
-                                 recurse ? -1 : 0,
-                                 ctx->cancel_func, ctx->cancel_baton,
-                                 pool));
-  SVN_ERR(svn_wc_entry(&entry, wcpath, adm_access, FALSE, pool));
-  if (! entry)
-    {
-      return svn_error_createf(SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                               _("Cannot read entry for '%s'"), wcpath);
-    }
+  int adm_lock_level = SVN_WC__LEVELS_TO_LOCK_FROM_DEPTH(depth);
+
+  SVN_ERR(svn_wc_adm_probe_open3(&adm_access, NULL, wcpath, FALSE, 
+                                 adm_lock_level, ctx->cancel_func, 
+                                 ctx->cancel_baton, pool));
+  SVN_ERR(svn_wc__entry_versioned(&entry, wcpath, adm_access, FALSE, pool));
 
-  SVN_ERR(build_info_from_entry(&info, entry, pool));
-  fe_baton.receiver = receiver;
-  fe_baton.receiver_baton = receiver_baton;
 
   if (entry->kind == svn_node_file)
-    return receiver(receiver_baton, wcpath, info, pool);
-
+    {
+      if (SVN_WC__CL_MATCH(changelist_hash, entry))
+        {
+          svn_info_t *info;
+          SVN_ERR(build_info_from_entry(&info, entry, pool));
+          return receiver(receiver_baton, wcpath, info, pool);
+        }
+    }
   else if (entry->kind == svn_node_dir)
     {
-      if (recurse)
-        SVN_ERR(svn_wc_walk_entries2(wcpath, adm_access,
-                                     &entry_walk_callbacks, &fe_baton,
-                                     FALSE, ctx->cancel_func,
-                                     ctx->cancel_baton, pool));
-      else
-        return receiver(receiver_baton, wcpath, info, pool);
+      struct found_entry_baton fe_baton;
+      fe_baton.changelist_hash = changelist_hash;
+      fe_baton.receiver = receiver;
+      fe_baton.receiver_baton = receiver_baton;
+      SVN_ERR(svn_wc_walk_entries3(wcpath, adm_access,
+                                   &entry_walk_callbacks, &fe_baton,
+                                   depth, FALSE, ctx->cancel_func,
+                                   ctx->cancel_baton, pool));
     }
-      
+
   return SVN_NO_ERROR;
 }
 
@@ -291,7 +317,7 @@ same_resource_in_head(svn_boolean_t *same_p,
                                     url, &peg_rev,
                                     &start_rev, &end_rev,
                                     ctx, pool);
-  if (err && 
+  if (err &&
       ((err->apr_err == SVN_ERR_CLIENT_UNRELATED_RESOURCES) ||
        (err->apr_err == SVN_ERR_FS_NOT_FOUND)))
     {
@@ -314,14 +340,15 @@ same_resource_in_head(svn_boolean_t *same_p,
 }
 
 svn_error_t *
-svn_client_info(const char *path_or_url,
-                const svn_opt_revision_t *peg_revision,
-                const svn_opt_revision_t *revision,
-                svn_info_receiver_t receiver,
-                void *receiver_baton,
-                svn_boolean_t recurse,
-                svn_client_ctx_t *ctx,
-                apr_pool_t *pool)
+svn_client_info2(const char *path_or_url,
+                 const svn_opt_revision_t *peg_revision,
+                 const svn_opt_revision_t *revision,
+                 svn_info_receiver_t receiver,
+                 void *receiver_baton,
+                 svn_depth_t depth,
+                 const apr_array_header_t *changelists,
+                 svn_client_ctx_t *ctx,
+                 apr_pool_t *pool)
 {
   svn_ra_session_t *ra_session, *parent_ra_session;
   svn_revnum_t rev;
@@ -335,16 +362,19 @@ svn_client_info(const char *path_or_url,
   svn_dirent_t *the_ent;
   svn_info_t *info;
   svn_error_t *err;
-   
-  if ((revision == NULL 
+
+  if ((revision == NULL
        || revision->kind == svn_opt_revision_unspecified)
       && (peg_revision == NULL
           || peg_revision->kind == svn_opt_revision_unspecified))
     {
       /* Do all digging in the working copy. */
-      return crawl_entries(path_or_url,
-                           receiver, receiver_baton,               
-                           recurse, ctx, pool);
+      apr_hash_t *changelist_hash = NULL;
+      if (changelists && changelists->nelts)
+        SVN_ERR(svn_hash_from_cstring_keys(&changelist_hash, 
+                                           changelists, pool));
+      return crawl_entries(path_or_url, receiver, receiver_baton,
+                           depth, changelist_hash, ctx, pool);
     }
 
   /* Go repository digging instead. */
@@ -353,15 +383,16 @@ svn_client_info(const char *path_or_url,
      return RA session to the possibly-renamed URL as it exists in REVISION.
      The ra_session returned will be anchored on this "final" URL. */
   SVN_ERR(svn_client__ra_session_from_path(&ra_session, &rev,
-                                           &url, path_or_url, peg_revision,
+                                           &url, path_or_url, NULL,
+                                           peg_revision,
                                            revision, ctx, pool));
 
-  SVN_ERR(svn_ra_get_repos_root(ra_session, &repos_root_URL, pool));
-  SVN_ERR(svn_ra_get_uuid(ra_session, &repos_UUID, pool));
-  
+  SVN_ERR(svn_ra_get_repos_root2(ra_session, &repos_root_URL, pool));
+  SVN_ERR(svn_ra_get_uuid2(ra_session, &repos_UUID, pool));
+
   svn_path_split(url, &parent_url, &base_name, pool);
   base_name = svn_path_uri_decode(base_name, pool);
-  
+
   /* Get the dirent for the URL itself. */
   err = svn_ra_stat(ra_session, "", rev, &the_ent, pool);
 
@@ -373,45 +404,52 @@ svn_client_info(const char *path_or_url,
       /* Fall back to pre-1.2 strategy for fetching dirent's URL. */
       svn_error_clear(err);
 
-      SVN_ERR(svn_ra_check_path(ra_session, "", rev, &url_kind, pool));      
-      if (url_kind == svn_node_none)
-        return svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
-                                 _("URL '%s' non-existent in revision '%ld'"),
-                                 url, rev);
-
       if (strcmp(url, repos_root_URL) == 0)
         {
           /* In this universe, there's simply no way to fetch
-             information about the repository's root directory!  So
-             degrade gracefully.  Rather than throw error, return no
-             information about the repos root, but at least give
-             recursion a chance. */     
-          goto recurse;
-        }        
-      
+             information about the repository's root directory!
+             If we're recursing, degrade gracefully: rather than
+             throw an error, return no information about the
+             repos root. */
+          if (depth > svn_depth_empty)
+            goto pre_1_2_recurse;
+
+          /* Otherwise, we really are stuck.  Better tell the user
+             what's going on. */
+          return svn_error_createf(SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                                   _("Server does not support retrieving "
+                                     "information about the repository root"));
+        }
+
+      SVN_ERR(svn_ra_check_path(ra_session, "", rev, &url_kind, pool));
+      if (url_kind == svn_node_none)
+        return svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
+                                 _("URL '%s' non-existent in revision %ld"),
+                                 url, rev);
+
       /* Open a new RA session to the item's parent. */
       SVN_ERR(svn_client__open_ra_session_internal(&parent_ra_session,
                                                    parent_url, NULL,
-                                                   NULL, NULL, FALSE, TRUE, 
+                                                   NULL, NULL, FALSE, TRUE,
                                                    ctx, pool));
-      
+
       /* Get all parent's entries, and find the item's dirent in the hash. */
       SVN_ERR(svn_ra_get_dir2(parent_ra_session, &parent_ents, NULL, NULL,
                               "", rev, DIRENT_FIELDS, pool));
       the_ent = apr_hash_get(parent_ents, base_name, APR_HASH_KEY_STRING);
       if (the_ent == NULL)
         return svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
-                                 _("URL '%s' non-existent in revision '%ld'"),
+                                 _("URL '%s' non-existent in revision %ld"),
                                  url, rev);
     }
   else if (err)
     {
       return err;
     }
-  
+
   if (! the_ent)
     return svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
-                             _("URL '%s' non-existent in revision '%ld'"),
+                             _("URL '%s' non-existent in revision %ld"),
                              url, rev);
 
   /* Check if the URL exists in HEAD and refers to the same resource.
@@ -441,24 +479,23 @@ svn_client_info(const char *path_or_url,
   else
     lock = NULL;
 
-  /* Push the URL's dirent (and lock) at the callback.*/  
+  /* Push the URL's dirent (and lock) at the callback.*/
   SVN_ERR(build_info_from_dirent(&info, the_ent, lock, url, rev,
                                  repos_UUID, repos_root_URL, pool));
   SVN_ERR(receiver(receiver_baton, base_name, info, pool));
 
-  /* Possibly recurse, using the original RA session. */  
- recurse:
-  
-  if (recurse && (the_ent->kind == svn_node_dir))
+  /* Possibly recurse, using the original RA session. */
+  if (depth > svn_depth_empty && (the_ent->kind == svn_node_dir))
     {
       apr_hash_t *locks;
 
+pre_1_2_recurse:
       if (peg_revision->kind == svn_opt_revision_head)
         {
           err = svn_ra_get_locks(ra_session, &locks, "", pool);
-          
+
           /* Catch specific errors thrown by old mod_dav_svn or svnserve. */
-          if (err && 
+          if (err &&
               (err->apr_err == SVN_ERR_RA_NOT_IMPLEMENTED
                || err->apr_err == SVN_ERR_UNSUPPORTED_FEATURE))
             {
@@ -470,17 +507,33 @@ svn_client_info(const char *path_or_url,
         }
       else
         locks = apr_hash_make(pool); /* use an empty hash */
-        
+
       SVN_ERR(push_dir_info(ra_session, url, "", rev,
                             repos_UUID, repos_root_URL,
                             receiver, receiver_baton,
-                            ctx, locks, pool));
+                            depth, ctx, locks, pool));
     }
 
   return SVN_NO_ERROR;
 }
 
 
+svn_error_t *
+svn_client_info(const char *path_or_url,
+                const svn_opt_revision_t *peg_revision,
+                const svn_opt_revision_t *revision,
+                svn_info_receiver_t receiver,
+                void *receiver_baton,
+                svn_boolean_t recurse,
+                svn_client_ctx_t *ctx,
+                apr_pool_t *pool)
+{
+  return svn_client_info2(path_or_url, peg_revision, revision,
+                          receiver, receiver_baton,
+                          SVN_DEPTH_INFINITY_OR_EMPTY(recurse), 
+                          NULL, ctx, pool);
+}
+
 svn_info_t *
 svn_info_dup(const svn_info_t *info, apr_pool_t *pool)
 {
