@@ -2,7 +2,7 @@
  * repos_diff.c -- The diff editor for comparing two repository versions
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -27,6 +27,7 @@
  * each file have been created the diff callback is invoked to display
  * the difference between the two files.  */
 
+#include "svn_hash.h"
 #include "svn_wc.h"
 #include "svn_pools.h"
 #include "svn_path.h"
@@ -49,9 +50,6 @@ struct edit_baton {
   const svn_wc_diff_callbacks2_t *diff_callbacks;
   void *diff_cmd_baton;
 
-  /* RECURSE is TRUE if this is a recursive diff or merge, false otherwise */
-  svn_boolean_t recurse;
-
   /* DRY_RUN is TRUE if this is a dry-run diff, false otherwise. */
   svn_boolean_t dry_run;
 
@@ -65,13 +63,19 @@ struct edit_baton {
      set_target_revision(). */
   svn_revnum_t target_revision;
 
-  /* A temporary empty file. Used for add/delete differences. This is
-     cached here so that it can be reused, all empty files are the same. */
+  /* The path to a temporary empty file used for add/delete
+     differences.  The path is cached here so that it can be reused,
+     since all empty files are the same. */
   const char *empty_file;
 
   /* Empty hash used for adds. */
   apr_hash_t *empty_hash;
 
+  /* Hash used to check replaced paths. Key is path relative CWD,
+   * Value is *kind_action_state_t.
+   * All allocations are from edit_baton's pool. */
+  apr_hash_t *deleted_paths;
+
   /* If the func is non-null, send notifications of actions. */
   svn_wc_notify_func2_t notify_func;
   void *notify_baton;
@@ -79,6 +83,13 @@ struct edit_baton {
   apr_pool_t *pool;
 };
 
+typedef struct kind_action_state_t
+{
+  svn_node_kind_t kind;
+  svn_wc_notify_action_t action;
+  svn_wc_notify_state_t state;
+} kind_action_state_t;
+
 /* Directory level baton.
  */
 struct dir_baton {
@@ -251,7 +262,7 @@ get_file_mime_types(const char **mimetype1,
  * delete this file.
  */
 static svn_error_t *
-get_file_from_ra(struct file_baton *b)
+get_file_from_ra(struct file_baton *b, svn_revnum_t revision)
 {
   apr_file_t *file;
   svn_stream_t *fstream;
@@ -266,7 +277,7 @@ get_file_from_ra(struct file_baton *b)
   fstream = svn_stream_from_aprfile(file, b->pool);
   SVN_ERR(svn_ra_get_file(b->edit_baton->ra_session,
                           b->path,
-                          b->edit_baton->revision,
+                          revision,
                           fstream, NULL,
                           &(b->pristine_props),
                           b->pool));
@@ -275,14 +286,14 @@ get_file_from_ra(struct file_baton *b)
   return SVN_NO_ERROR;
 }
 
-/* Get the props attached to a directory in the repository. */
+/* Get the props attached to a directory in the repository at BASE_REVISION. */
 static svn_error_t *
-get_dirprops_from_ra(struct dir_baton *b)
+get_dirprops_from_ra(struct dir_baton *b, svn_revnum_t base_revision)
 {
   SVN_ERR(svn_ra_get_dir2(b->edit_baton->ra_session,
                           NULL, NULL, &(b->pristine_props),
                           b->path,
-                          b->edit_baton->revision,
+                          base_revision,
                           0,
                           b->pool));
 
@@ -290,21 +301,21 @@ get_dirprops_from_ra(struct dir_baton *b)
 }
 
 
-/* Create an empty file, the path to the file is returned in EMPTY_FILE.
- * If ADM_ACCESS is not NULL and a lock is held, create the file in the
- * adm tmp/ area, otherwise use a system temp dir.
- *
- * If FILE is non-NULL, an open file is returned in *FILE.
- */
+/* Create an empty file, the path to the file is returned in
+   EMPTY_FILE_PATH.  If ADM_ACCESS is not NULL and a lock is held,
+   create the file in the adm tmp/ area, otherwise use a system temp
+   directory.
+
+   If FILE is non-NULL, an open file is returned in *FILE. */
 static svn_error_t *
 create_empty_file(apr_file_t **file,
-                  const char **empty_file,
+                  const char **empty_file_path,
                   svn_wc_adm_access_t *adm_access,
                   svn_io_file_del_t delete_when,
                   apr_pool_t *pool)
 {
   if (adm_access && svn_wc_adm_locked(adm_access))
-    SVN_ERR(svn_wc_create_tmp_file2(file, empty_file,
+    SVN_ERR(svn_wc_create_tmp_file2(file, empty_file_path,
                                     svn_wc_adm_access_path(adm_access),
                                     delete_when, pool));
   else
@@ -312,7 +323,7 @@ create_empty_file(apr_file_t **file,
       const char *temp_dir;
 
       SVN_ERR(svn_io_temp_dir(&temp_dir, pool));
-      SVN_ERR(svn_io_open_unique_file2(file, empty_file,
+      SVN_ERR(svn_io_open_unique_file2(file, empty_file_path,
                                        svn_path_join(temp_dir, "tmp", pool),
                                        "", delete_when, pool));
     }
@@ -349,7 +360,7 @@ get_path_access(svn_wc_adm_access_t **path_access,
 
   return SVN_NO_ERROR;
 }
-                  
+
 /* Like get_path_access except the returned access baton, in
    *PARENT_ACCESS, is for the parent of PATH rather than for PATH
    itself. */
@@ -375,30 +386,30 @@ get_parent_access(svn_wc_adm_access_t **parent_access,
  * that it can be reused, all empty files are the same.
  */
 static svn_error_t *
-get_empty_file(struct edit_baton *b,
-               const char **empty_file)
+get_empty_file(struct edit_baton *eb,
+               const char **empty_file_path)
 {
   /* Create the file if it does not exist */
   /* Note that we tried to use /dev/null in r17220, but
      that won't work on Windows: it's impossible to stat NUL */
-  if (!b->empty_file)
-    SVN_ERR(create_empty_file(NULL, &(b->empty_file), b->adm_access,
-                              svn_io_file_del_on_pool_cleanup, b->pool));
+  if (!eb->empty_file)
+    SVN_ERR(create_empty_file(NULL, &(eb->empty_file), eb->adm_access,
+                              svn_io_file_del_on_pool_cleanup, eb->pool));
 
 
-  *empty_file = b->empty_file;
+  *empty_file_path = eb->empty_file;
 
   return SVN_NO_ERROR;
 }
 
 /* An editor function. The root of the comparison hierarchy */
 static svn_error_t *
-set_target_revision(void *edit_baton, 
+set_target_revision(void *edit_baton,
                     svn_revnum_t target_revision,
                     apr_pool_t *pool)
 {
   struct edit_baton *eb = edit_baton;
-  
+
   eb->target_revision = target_revision;
   return SVN_NO_ERROR;
 }
@@ -414,9 +425,9 @@ open_root(void *edit_baton,
   struct dir_baton *b = make_dir_baton("", NULL, eb, FALSE, pool);
 
   /* Override the wcpath in our baton. */
-  b->wcpath = eb->target ? apr_pstrdup(pool, eb->target) : "";
+  b->wcpath = apr_pstrdup(pool, eb->target);
 
-  SVN_ERR(get_dirprops_from_ra(b));
+  SVN_ERR(get_dirprops_from_ra(b, base_revision));
 
   *root_baton = b;
   return SVN_NO_ERROR;
@@ -437,11 +448,7 @@ delete_entry(const char *path,
   svn_wc_notify_action_t action = svn_wc_notify_skip;
 
   /* We need to know if this is a directory or a file */
-  SVN_ERR(svn_ra_check_path(pb->edit_baton->ra_session,
-                            path,
-                            pb->edit_baton->revision,
-                            &kind,
-                            pool));
+  SVN_ERR(svn_ra_check_path(eb->ra_session, path, eb->revision, &kind, pool));
   SVN_ERR(get_path_access(&adm_access, eb->adm_access, pb->wcpath,
                           TRUE, pool));
   if ((! eb->adm_access) || adm_access)
@@ -452,51 +459,63 @@ delete_entry(const char *path,
           {
             const char *mimetype1, *mimetype2;
             struct file_baton *b;
-            
+
             /* Compare a file being deleted against an empty file */
-            b = make_file_baton(path, FALSE, pb->edit_baton, pool);
-            SVN_ERR(get_file_from_ra(b));
+            b = make_file_baton(path, FALSE, eb, pool);
+            SVN_ERR(get_file_from_ra(b, eb->revision));
             SVN_ERR(get_empty_file(b->edit_baton, &(b->path_end_revision)));
-            
+
             get_file_mime_types(&mimetype1, &mimetype2, b);
-            
-            SVN_ERR(pb->edit_baton->diff_callbacks->file_deleted 
+
+            SVN_ERR(eb->diff_callbacks->file_deleted
                     (adm_access, &state, b->wcpath,
                      b->path_start_revision,
                      b->path_end_revision,
                      mimetype1, mimetype2,
                      b->pristine_props,
                      b->edit_baton->diff_cmd_baton));
-            
+
             break;
           }
         case svn_node_dir:
           {
-            SVN_ERR(pb->edit_baton->diff_callbacks->dir_deleted 
-                    (adm_access, &state, 
+            SVN_ERR(eb->diff_callbacks->dir_deleted
+                    (adm_access, &state,
                      svn_path_join(eb->target, path, pool),
-                     pb->edit_baton->diff_cmd_baton));
+                     eb->diff_cmd_baton));
             break;
           }
         default:
           break;
         }
-      
+
       if ((state != svn_wc_notify_state_missing)
           && (state != svn_wc_notify_state_obstructed))
-        action = svn_wc_notify_update_delete;
+        {
+          action = svn_wc_notify_update_delete;
+          if (eb->dry_run)
+            {
+              /* Remember what we _would've_ deleted (issue #2584). */
+              const char *wcpath = svn_path_join(eb->target, path, pb->pool);
+              apr_hash_set(svn_client__dry_run_deletions(eb->diff_cmd_baton),
+                           wcpath, APR_HASH_KEY_STRING, wcpath);
+
+              /* ### TODO: if (kind == svn_node_dir), record all
+                 ### children as deleted to avoid collisions from
+                 ### subsequent edits. */
+            }
+        }
     }
 
-  if (pb->edit_baton->notify_func)
+  if (eb->notify_func)
     {
-      svn_wc_notify_t *notify
-        = svn_wc_create_notify(svn_path_join(eb->target, path, pool),
-                               action, pool);
-      notify->kind = kind;
-      notify->content_state = notify->prop_state = state;
-      notify->lock_state = svn_wc_notify_lock_state_inapplicable;
-      (*pb->edit_baton->notify_func)(pb->edit_baton->notify_baton, notify,
-                                     pool);
+      const char* deleted_path;
+      kind_action_state_t *kas = apr_palloc(eb->pool, sizeof(*kas));
+      deleted_path = svn_path_join(eb->target, path, eb->pool);
+      kas->kind = kind;
+      kas->action = action;
+      kas->state = state;
+      apr_hash_set(eb->deleted_paths, deleted_path, APR_HASH_KEY_STRING, kas);
     }
   return SVN_NO_ERROR;
 }
@@ -519,17 +538,16 @@ add_directory(const char *path,
 
   /* ### TODO: support copyfrom? */
 
-  b = make_dir_baton(path, pb, pb->edit_baton, TRUE, pool);
-  b->pristine_props = pb->edit_baton->empty_hash;
+  b = make_dir_baton(path, pb, eb, TRUE, pool);
+  b->pristine_props = eb->empty_hash;
   *child_baton = b;
 
-  SVN_ERR(get_path_access(&adm_access,
-                          pb->edit_baton->adm_access, pb->wcpath,
-                          TRUE, pool));
+  SVN_ERR(get_path_access(&adm_access, eb->adm_access, pb->wcpath, TRUE,
+                          pool));
 
-  SVN_ERR(pb->edit_baton->diff_callbacks->dir_added 
+  SVN_ERR(eb->diff_callbacks->dir_added
           (adm_access, &state, b->wcpath, eb->target_revision,
-           pb->edit_baton->diff_cmd_baton));
+           eb->diff_cmd_baton));
 
   if ((state == svn_wc_notify_state_missing)
       || (state == svn_wc_notify_state_obstructed))
@@ -537,12 +555,38 @@ add_directory(const char *path,
   else
     action = svn_wc_notify_update_add;
 
-  if (pb->edit_baton->notify_func)
+  if (eb->notify_func)
     {
-      svn_wc_notify_t *notify = svn_wc_create_notify(b->wcpath, action, pool);
-      notify->kind = svn_node_dir;
-      (*pb->edit_baton->notify_func)(pb->edit_baton->notify_baton, notify,
-                                     pool);
+      svn_wc_notify_t *notify;
+      svn_boolean_t is_replace = FALSE;
+      kind_action_state_t *kas = apr_hash_get(eb->deleted_paths, b->wcpath,
+                                              APR_HASH_KEY_STRING);
+      if (kas)
+        {
+          svn_wc_notify_action_t new_action;
+          if (kas->action == svn_wc_notify_update_delete
+              && action == svn_wc_notify_update_add)
+            {
+              is_replace = TRUE;
+              new_action = svn_wc_notify_update_replace;
+            }
+          else
+            new_action = kas->action;
+          notify  = svn_wc_create_notify(b->wcpath, new_action, pool);
+          notify->kind = kas->kind;
+          notify->content_state = notify->prop_state = kas->state;
+          notify->lock_state = svn_wc_notify_lock_state_inapplicable;
+          (*eb->notify_func)(eb->notify_baton, notify, pool);
+          apr_hash_set(eb->deleted_paths, b->wcpath,
+                       APR_HASH_KEY_STRING, NULL);
+        }
+
+      if (!is_replace)
+        {
+          notify = svn_wc_create_notify(b->wcpath, action, pool);
+          notify->kind = svn_node_dir;
+          (*eb->notify_func)(eb->notify_baton, notify, pool);
+        }
     }
 
   return SVN_NO_ERROR;
@@ -562,7 +606,7 @@ open_directory(const char *path,
   b = make_dir_baton(path, pb, pb->edit_baton, FALSE, pool);
   *child_baton = b;
 
-  SVN_ERR(get_dirprops_from_ra(b));
+  SVN_ERR(get_dirprops_from_ra(b, base_revision));
 
   return SVN_NO_ERROR;
 }
@@ -605,7 +649,7 @@ open_file(const char *path,
   b = make_file_baton(path, FALSE, pb->edit_baton, pool);
   *file_baton = b;
 
-  SVN_ERR(get_file_from_ra(b));
+  SVN_ERR(get_file_from_ra(b, base_revision));
 
   return SVN_NO_ERROR;
 }
@@ -701,12 +745,12 @@ close_file(void *file_baton,
     content_state = svn_wc_notify_state_unknown,
     prop_state = svn_wc_notify_state_unknown;
 
-  err = get_parent_access(&adm_access, eb->adm_access, 
+  err = get_parent_access(&adm_access, eb->adm_access,
                           b->wcpath, eb->dry_run, b->pool);
 
   if (err && err->apr_err == SVN_ERR_WC_NOT_LOCKED)
     {
-      /* ### maybe try to stat the local b->wcpath? */      
+      /* ### maybe try to stat the local b->wcpath? */
       /* If the file path doesn't exist, then send a 'skipped' notification. */
       if (eb->notify_func)
         {
@@ -718,7 +762,7 @@ close_file(void *file_baton,
           notify->prop_state = prop_state;
           (*eb->notify_func)(eb->notify_baton, notify, pool);
         }
-      
+
       svn_error_clear(err);
       return SVN_NO_ERROR;
     }
@@ -765,12 +809,38 @@ close_file(void *file_baton,
 
   if (eb->notify_func)
     {
-      svn_wc_notify_t *notify = svn_wc_create_notify(b->wcpath, action,
-                                                     pool);
-      notify->kind = svn_node_file;
-      notify->content_state = content_state;
-      notify->prop_state = prop_state;
-      (*eb->notify_func)(eb->notify_baton, notify, pool);
+      svn_wc_notify_t *notify;
+      svn_boolean_t is_replace = FALSE;
+      kind_action_state_t *kas = apr_hash_get(eb->deleted_paths, b->wcpath,
+                                              APR_HASH_KEY_STRING);
+      if (kas)
+        {
+          svn_wc_notify_action_t new_action;
+          if (kas->action == svn_wc_notify_update_delete
+              && action == svn_wc_notify_update_add)
+            {
+              is_replace = TRUE;
+              new_action = svn_wc_notify_update_replace;
+            }
+          else
+            new_action = kas->action;
+          notify  = svn_wc_create_notify(b->wcpath, new_action, pool);
+          notify->kind = kas->kind;
+          notify->content_state = notify->prop_state = kas->state;
+          notify->lock_state = svn_wc_notify_lock_state_inapplicable;
+          (*eb->notify_func)(eb->notify_baton, notify, pool);
+          apr_hash_set(eb->deleted_paths, b->wcpath,
+                       APR_HASH_KEY_STRING, NULL);
+        }
+
+      if (!is_replace)
+        {
+          notify = svn_wc_create_notify(b->wcpath, action, pool);
+          notify->kind = svn_node_file;
+          notify->content_state = content_state;
+          notify->prop_state = prop_state;
+          (*eb->notify_func)(eb->notify_baton, notify, pool);
+        }
     }
 
   return SVN_NO_ERROR;
@@ -786,6 +856,9 @@ close_directory(void *dir_baton,
   svn_wc_notify_state_t prop_state = svn_wc_notify_state_unknown;
   svn_error_t *err;
 
+  if (eb->dry_run)
+    svn_hash__clear(svn_client__dry_run_deletions(eb->diff_cmd_baton));
+
   if (b->propchanges->nelts > 0)
     {
       svn_wc_adm_access_t *adm_access;
@@ -794,7 +867,7 @@ close_directory(void *dir_baton,
 
       if (err && err->apr_err == SVN_ERR_WC_NOT_LOCKED)
         {
-          /* ### maybe try to stat the local b->wcpath? */          
+          /* ### maybe try to stat the local b->wcpath? */
           /* If the path doesn't exist, then send a 'skipped' notification. */
           if (eb->notify_func)
             {
@@ -805,7 +878,7 @@ close_directory(void *dir_baton,
                 = svn_wc_notify_state_missing;
               (*eb->notify_func)(eb->notify_baton, notify, pool);
             }
-          svn_error_clear(err);      
+          svn_error_clear(err);
           return SVN_NO_ERROR;
         }
       else if (err)
@@ -827,8 +900,26 @@ close_directory(void *dir_baton,
      isn't getting all the information? */
   if (!b->added && eb->notify_func)
     {
-      svn_wc_notify_t *notify
-        = svn_wc_create_notify(b->wcpath, svn_wc_notify_update_update, pool);
+      svn_wc_notify_t *notify;
+      apr_hash_index_t *hi;
+
+      for (hi = apr_hash_first(NULL, eb->deleted_paths); hi;
+           hi = apr_hash_next(hi))
+        {
+          const void *deleted_path;
+          kind_action_state_t *kas;
+          apr_hash_this(hi, &deleted_path, NULL, (void *)&kas);
+          notify  = svn_wc_create_notify(deleted_path, kas->action, pool);
+          notify->kind = kas->kind;
+          notify->content_state = notify->prop_state = kas->state;
+          notify->lock_state = svn_wc_notify_lock_state_inapplicable;
+          (*eb->notify_func)(eb->notify_baton, notify, pool);
+          apr_hash_set(eb->deleted_paths, deleted_path,
+                       APR_HASH_KEY_STRING, NULL);
+        }
+
+      notify = svn_wc_create_notify(b->wcpath,
+                                    svn_wc_notify_update_update, pool);
       notify->kind = svn_node_dir;
       notify->content_state = svn_wc_notify_state_inapplicable;
       notify->prop_state = prop_state;
@@ -853,7 +944,7 @@ change_file_prop(void *file_baton,
   propchange = apr_array_push(b->propchanges);
   propchange->name = apr_pstrdup(b->pool, name);
   propchange->value = value ? svn_string_dup(value, b->pool) : NULL;
-  
+
   return SVN_NO_ERROR;
 }
 
@@ -887,13 +978,64 @@ close_edit(void *edit_baton,
   return SVN_NO_ERROR;
 }
 
+/* An editor function.  */
+static svn_error_t *
+absent_directory(const char *path,
+                 void *parent_baton,
+                 apr_pool_t *pool)
+{
+  struct dir_baton *pb = parent_baton;
+  struct edit_baton *eb = pb->edit_baton;
+
+  if (eb->notify_func)
+    {
+      svn_wc_notify_t *notify
+        = svn_wc_create_notify(svn_path_join(pb->wcpath,
+                                             svn_path_basename(path, pool),
+                                             pool),
+                               svn_wc_notify_skip, pool);
+      notify->kind = svn_node_dir;
+      notify->content_state = notify->prop_state
+        = svn_wc_notify_state_missing;
+      (*eb->notify_func)(eb->notify_baton, notify, pool);
+    }
+
+  return SVN_NO_ERROR;
+}
+
+
+/* An editor function.  */
+static svn_error_t *
+absent_file(const char *path,
+            void *parent_baton,
+            apr_pool_t *pool)
+{
+  struct dir_baton *pb = parent_baton;
+  struct edit_baton *eb = pb->edit_baton;
+
+  if (eb->notify_func)
+    {
+      svn_wc_notify_t *notify
+        = svn_wc_create_notify(svn_path_join(pb->wcpath,
+                                             svn_path_basename(path, pool),
+                                             pool),
+                               svn_wc_notify_skip, pool);
+      notify->kind = svn_node_file;
+      notify->content_state = notify->prop_state
+        = svn_wc_notify_state_missing;
+      (*eb->notify_func)(eb->notify_baton, notify, pool);
+    }
+
+  return SVN_NO_ERROR;
+}
+
 /* Create a repository diff editor and baton.  */
 svn_error_t *
 svn_client__get_diff_editor(const char *target,
                             svn_wc_adm_access_t *adm_access,
                             const svn_wc_diff_callbacks2_t *diff_callbacks,
                             void *diff_cmd_baton,
-                            svn_boolean_t recurse,
+                            svn_depth_t depth,
                             svn_boolean_t dry_run,
                             svn_ra_session_t *ra_session,
                             svn_revnum_t revision,
@@ -913,12 +1055,12 @@ svn_client__get_diff_editor(const char *target,
   eb->adm_access = adm_access;
   eb->diff_callbacks = diff_callbacks;
   eb->diff_cmd_baton = diff_cmd_baton;
-  eb->recurse = recurse;
   eb->dry_run = dry_run;
   eb->ra_session = ra_session;
   eb->revision = revision;
   eb->empty_file = NULL;
   eb->empty_hash = apr_hash_make(subpool);
+  eb->deleted_paths = apr_hash_make(subpool);
   eb->pool = subpool;
   eb->notify_func = notify_func;
   eb->notify_baton = notify_baton;
@@ -936,6 +1078,8 @@ svn_client__get_diff_editor(const char *target,
   tree_editor->change_file_prop = change_file_prop;
   tree_editor->change_dir_prop = change_dir_prop;
   tree_editor->close_edit = close_edit;
+  tree_editor->absent_directory = absent_directory;
+  tree_editor->absent_file = absent_file;
 
   SVN_ERR(svn_delta_get_cancellation_editor(cancel_func,
                                             cancel_baton,
@@ -945,5 +1089,6 @@ svn_client__get_diff_editor(const char *target,
                                             edit_baton,
                                             pool));
 
+  /* We don't destroy subpool, as it's managed by the edit baton. */
   return SVN_NO_ERROR;
 }
