@@ -118,7 +118,7 @@ open_root(void *edit_baton,
           void **root_baton)
 {
   struct item_baton *ib = create_item_baton(edit_baton, "",
-                                            svn_node_dir, pool);;
+                                            svn_node_dir, pool);
 
   *root_baton = ib;
   return SVN_NO_ERROR;
@@ -198,7 +198,7 @@ close_directory(void *dir_baton,
 {
   struct item_baton *ib = dir_baton;
   struct edit_baton *eb = ib->edit_baton;
-  
+
   if (ib->summarize)
     SVN_ERR(eb->summarize_func(ib->summarize, eb->summarize_func_baton,
                                pool));
