@@ -1,7 +1,7 @@
 /* reps-strings.c : intepreting representations with respect to strings
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -118,11 +118,10 @@ delta_string_keys(apr_array_header_t **keys,
   /* Now, push the string keys for each window into *KEYS */
   for (i = 0; i < chunks->nelts; i++)
     {
-      rep_delta_chunk_t *chunk =
-        (((rep_delta_chunk_t **) chunks->elts)[i]);
+      rep_delta_chunk_t *chunk = APR_ARRAY_IDX(chunks, i, rep_delta_chunk_t *);
 
       key = apr_pstrdup(pool, chunk->string_key);
-      (*((const char **)(apr_array_push(*keys)))) = key;
+      APR_ARRAY_PUSH(*keys, const char *) = key;
     }
 
   return SVN_NO_ERROR;
@@ -143,7 +142,7 @@ delete_strings(apr_array_header_t *keys,
   for (i = 0; i < keys->nelts; i++)
     {
       svn_pool_clear(subpool);
-      str_key = ((const char **) keys->elts)[i];
+      str_key = APR_ARRAY_IDX(keys, i, const char *);
       SVN_ERR(svn_fs_bdb__string_delete(fs, str_key, trail, subpool));
     }
   svn_pool_destroy(subpool);
@@ -299,7 +298,7 @@ get_one_window(struct compose_handler_baton *cb,
     {
       amt = sizeof(diffdata);
       SVN_ERR(svn_fs_bdb__string_read(fs, str_key, diffdata,
-                                      off, &amt, cb->trail, 
+                                      off, &amt, cb->trail,
                                       cb->trail->pool));
       off += amt;
       SVN_ERR(svn_stream_write(wstream, diffdata, &amt));
@@ -371,7 +370,7 @@ rep_undeltify_range(svn_fs_t *fs,
           source_buf = apr_palloc(cb.window_pool, source_len);
           SVN_ERR(svn_fs_bdb__string_read
                   (fs, fulltext->contents.fulltext.string_key,
-                   source_buf, cb.window->sview_offset, &source_len, 
+                   source_buf, cb.window->sview_offset, &source_len,
                    trail, pool));
           if (source_len != cb.window->sview_len)
             return svn_error_create
@@ -507,8 +506,8 @@ rep_read_range(svn_fs_t *fs,
                    rep_key);
 
               rep_key = chunk->rep_key;
-              *(representation_t**) apr_array_push(reps) = rep;
-              SVN_ERR(svn_fs_bdb__read_rep(&rep, fs, rep_key, 
+              APR_ARRAY_PUSH(reps, representation_t *) = rep;
+              SVN_ERR(svn_fs_bdb__read_rep(&rep, fs, rep_key,
                                            trail, pool));
             }
           while (rep->kind == rep_kind_delta
@@ -521,13 +520,13 @@ rep_read_range(svn_fs_t *fs,
 
           if (rep->kind == rep_kind_delta)
             rep = NULL;         /* Don't use source data */
-          
+
           err = rep_undeltify_range(fs, reps, rep, cur_chunk, buf,
                                     chunk_offset, len, trail, pool);
           if (err)
             {
               if (err->apr_err == SVN_ERR_FS_CORRUPT)
-                return svn_error_createf 
+                return svn_error_createf
                   (SVN_ERR_FS_CORRUPT, err,
                    _("Corruption detected whilst reading delta chain from "
                      "representation '%s' to '%s'"), first_rep_key, rep_key);
@@ -571,7 +570,7 @@ svn_fs_base__get_mutable_rep(const char **new_rep_key,
      we were provided was not mutable.  So, let's make a new
      representation and return its key to the caller. */
   SVN_ERR(svn_fs_bdb__string_append(fs, &new_str, 0, NULL, trail, pool));
-  rep = make_fulltext_rep(new_str, txn_id, 
+  rep = make_fulltext_rep(new_str, txn_id,
                           svn_md5_empty_string_digest(), pool);
   SVN_ERR(svn_fs_bdb__write_new_rep(new_rep_key, fs, rep, trail, pool));
 
@@ -594,7 +593,7 @@ svn_fs_base__delete_rep_if_mutable(svn_fs_t *fs,
 
   if (rep->kind == rep_kind_fulltext)
     {
-      SVN_ERR(svn_fs_bdb__string_delete(fs, 
+      SVN_ERR(svn_fs_bdb__string_delete(fs,
                                         rep->contents.fulltext.string_key,
                                         trail, pool));
     }
@@ -682,7 +681,7 @@ rep_read_get_baton(struct rep_read_baton **rb_p,
   apr_md5_init(&(b->md5_context));
 
   if (rep_key)
-    SVN_ERR(svn_fs_base__rep_contents_size(&(b->size), fs, rep_key, 
+    SVN_ERR(svn_fs_base__rep_contents_size(&(b->size), fs, rep_key,
                                            trail, pool));
   else
     b->size = 0;
@@ -854,7 +853,7 @@ txn_body_read_rep(void *baton, trail_t *trail)
                              args->rb->offset,
                              args->buf,
                              args->len,
-                             trail, 
+                             trail,
                              trail->pool));
 
       args->rb->offset += *(args->len);
@@ -890,7 +889,7 @@ txn_body_read_rep(void *baton, trail_t *trail)
               args->rb->checksum_finalized = TRUE;
 
               SVN_ERR(svn_fs_bdb__read_rep(&rep, args->rb->fs,
-                                           args->rb->rep_key, 
+                                           args->rb->rep_key,
                                            trail, trail->pool));
               if (! svn_md5_digests_match(checksum, rep->checksum))
                 return svn_error_createf
@@ -1027,7 +1026,7 @@ rep_write(svn_fs_t *fs,
   if (rep->kind == rep_kind_fulltext)
     {
       SVN_ERR(svn_fs_bdb__string_append
-              (fs, &(rep->contents.fulltext.string_key), len, buf, 
+              (fs, &(rep->contents.fulltext.string_key), len, buf,
                trail, pool));
     }
   else if (rep->kind == rep_kind_delta)
@@ -1070,7 +1069,7 @@ txn_body_write_rep(void *baton, trail_t *trail)
                     args->buf,
                     args->len,
                     args->wb->txn_id,
-                    trail, 
+                    trail,
                     trail->pool));
 
   apr_md5_update(&(args->wb->md5_context), args->buf, args->len);
@@ -1126,10 +1125,10 @@ txn_body_write_close_rep(void *baton, trail_t *trail)
   struct rep_write_baton *wb = baton;
   representation_t *rep;
 
-  SVN_ERR(svn_fs_bdb__read_rep(&rep, wb->fs, wb->rep_key, 
+  SVN_ERR(svn_fs_bdb__read_rep(&rep, wb->fs, wb->rep_key,
                                trail, trail->pool));
   memcpy(rep->checksum, wb->md5_digest, APR_MD5_DIGESTSIZE);
-  SVN_ERR(svn_fs_bdb__write_rep(wb->fs, wb->rep_key, rep, 
+  SVN_ERR(svn_fs_bdb__write_rep(wb->fs, wb->rep_key, rep,
                                 trail, trail->pool));
 
   return SVN_NO_ERROR;
@@ -1467,7 +1466,7 @@ svn_fs_base__rep_deltify(svn_fs_t *fs,
           ww->svndiff_len = new_target_baton.size;
           ww->text_off = tview_off;
           ww->text_len = window->tview_len;
-          (*((window_write_t **)(apr_array_push(windows)))) = ww;
+          APR_ARRAY_PUSH(windows, window_write_t *) = ww;
 
           /* Update our recordkeeping variables. */
           tview_off += window->tview_len;
@@ -1503,10 +1502,10 @@ svn_fs_base__rep_deltify(svn_fs_t *fs,
         svn_filesize_t old_size = 0;
 
         str_key = old_rep->contents.fulltext.string_key;
-        SVN_ERR(svn_fs_bdb__string_size(&old_size, fs, str_key, 
+        SVN_ERR(svn_fs_bdb__string_size(&old_size, fs, str_key,
                                         trail, pool));
         orig_str_keys = apr_array_make(pool, 1, sizeof(str_key));
-        (*((const char **)(apr_array_push(orig_str_keys)))) = str_key;
+        APR_ARRAY_PUSH(orig_str_keys, const char *) = str_key;
 
         /* If the new data is NOT an space optimization, destroy the
            string(s) we created, and get outta here. */
@@ -1515,7 +1514,7 @@ svn_fs_base__rep_deltify(svn_fs_t *fs,
             int i;
             for (i = 0; i < windows->nelts; i++)
               {
-                ww = ((window_write_t **) windows->elts)[i];
+                ww = APR_ARRAY_IDX(windows, i, window_write_t *);
                 SVN_ERR(svn_fs_bdb__string_delete(fs, ww->key, trail, pool));
               }
             return SVN_NO_ERROR;
@@ -1550,7 +1549,7 @@ svn_fs_base__rep_deltify(svn_fs_t *fs,
        chunks to the representation. */
     for (i = 0; i < windows->nelts; i++)
       {
-        ww = ((window_write_t **) windows->elts)[i];
+        ww = APR_ARRAY_IDX(windows, i, window_write_t *);
 
         /* Allocate a chunk and its window */
         chunk = apr_palloc(pool, sizeof(*chunk));
@@ -1563,7 +1562,7 @@ svn_fs_base__rep_deltify(svn_fs_t *fs,
         chunk->rep_key = source;
 
         /* Add this chunk to the array. */
-        (*((rep_delta_chunk_t **)(apr_array_push(chunks)))) = chunk;
+        APR_ARRAY_PUSH(chunks, rep_delta_chunk_t *) = chunk;
       }
 
     /* Put the chunks array into the representation. */
