@@ -44,12 +44,6 @@ struct dag_node_t
   /* The filesystem this dag node came from. */
   svn_fs_t *fs;
 
-  /* The pool in which this dag_node_t was allocated.  Unlike
-     filesystem and root pools, this is not a private pool for this
-     structure!  The caller may have allocated other objects of their
-     own in it.  */
-  apr_pool_t *pool;
-
   /* The node revision ID for this dag node, allocated in POOL.  */
   svn_fs_id_t *id;
 
@@ -105,7 +99,8 @@ svn_fs_fs__dag_get_fs(dag_node_t *node)
 }
 
 
-/* Dup NODEREV and all associated data into POOL */
+/* Dup NODEREV and all associated data into POOL.
+   Leaves the id and is_fresh_txn_root fields as zero bytes. */
 static node_revision_t *
 copy_node_revision(node_revision_t *noderev,
                    apr_pool_t *pool)
@@ -123,16 +118,16 @@ copy_node_revision(node_revision_t *noderev,
   nr->predecessor_count = noderev->predecessor_count;
   nr->data_rep = svn_fs_fs__rep_copy(noderev->data_rep, pool);
   nr->prop_rep = svn_fs_fs__rep_copy(noderev->prop_rep, pool);
-  
+  nr->mergeinfo_count = noderev->mergeinfo_count;
+  nr->has_mergeinfo = noderev->has_mergeinfo;
+
   if (noderev->created_path)
     nr->created_path = apr_pstrdup(pool, noderev->created_path);
   return nr;
 }
 
 
-/* Set *NODEREV_P to the cached node-revision for NODE.  If NODE is
-   immutable, the node-revision is allocated in NODE->pool.  If NODE
-   is mutable, the node-revision is allocated in POOL.
+/* Set *NODEREV_P to the cached node-revision for NODE in POOL.
 
    If you plan to change the contents of NODE, be careful!  We're
    handing you a pointer directly to our cached node-revision, not
@@ -155,15 +150,14 @@ get_node_revision(node_revision_t **noderev_p,
                                            node->id, pool));
       node->node_revision = noderev;
     }
-          
+
   /* Now NODE->node_revision is set.  */
   *noderev_p = node->node_revision;
   return SVN_NO_ERROR;
 }
 
 
-svn_boolean_t svn_fs_fs__dag_check_mutable(dag_node_t *node, 
-                                           const char *txn_id)
+svn_boolean_t svn_fs_fs__dag_check_mutable(dag_node_t *node)
 {
   return (svn_fs_fs__id_txn_id(svn_fs_fs__dag_get_id(node)) != NULL);
 }
@@ -181,8 +175,7 @@ svn_fs_fs__dag_get_node(dag_node_t **node,
   /* Construct the node. */
   new_node = apr_pcalloc(pool, sizeof(*new_node));
   new_node->fs = fs;
-  new_node->id = svn_fs_fs__id_copy(id, pool); 
-  new_node->pool = pool;
+  new_node->id = svn_fs_fs__id_copy(id, pool);
 
   /* Grab the contents so we can inspect the node's kind and created path. */
   SVN_ERR(get_node_revision(&noderev, new_node, pool));
@@ -226,7 +219,7 @@ svn_fs_fs__dag_get_predecessor_id(const svn_fs_id_t **id_p,
                                   apr_pool_t *pool)
 {
   node_revision_t *noderev;
-  
+
   SVN_ERR(get_node_revision(&noderev, node, pool));
   *id_p = noderev->predecessor_id;
   return SVN_NO_ERROR;
@@ -239,12 +232,58 @@ svn_fs_fs__dag_get_predecessor_count(int *count,
                                      apr_pool_t *pool)
 {
   node_revision_t *noderev;
-  
+
   SVN_ERR(get_node_revision(&noderev, node, pool));
   *count = noderev->predecessor_count;
   return SVN_NO_ERROR;
 }
 
+svn_error_t *
+svn_fs_fs__dag_get_mergeinfo_count(apr_int64_t *count,
+                                   dag_node_t *node,
+                                   apr_pool_t *pool)
+{
+  node_revision_t *noderev;
+
+  SVN_ERR(get_node_revision(&noderev, node, pool));
+  *count = noderev->mergeinfo_count;
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_fs_fs__dag_has_mergeinfo(svn_boolean_t *has_mergeinfo,
+                             dag_node_t *node,
+                             apr_pool_t *pool)
+{
+  node_revision_t *noderev;
+
+  SVN_ERR(get_node_revision(&noderev, node, pool));
+  *has_mergeinfo = noderev->has_mergeinfo;
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_fs_fs__dag_has_descendants_with_mergeinfo(svn_boolean_t *do_they,
+                                              dag_node_t *node,
+                                              apr_pool_t *pool)
+{
+  node_revision_t *noderev;
+
+  if (node->kind != svn_node_dir)
+    {
+      *do_they = FALSE;
+      return SVN_NO_ERROR;
+    }
+
+  SVN_ERR(get_node_revision(&noderev, node, pool));
+  if (noderev->mergeinfo_count > 1)
+    *do_they = TRUE;
+  else if (noderev->mergeinfo_count == 1 && !noderev->has_mergeinfo)
+    *do_they = TRUE;
+  else
+    *do_they = FALSE;
+  return SVN_NO_ERROR;
+}
 
 
 /*** Directory node functions ***/
@@ -252,25 +291,28 @@ svn_fs_fs__dag_get_predecessor_count(int *count,
 /* Some of these are helpers for functions outside this section. */
 
 /* Set *ID_P to the node-id for entry NAME in PARENT.  If no such
-   entry, set *ID_P to NULL but do not error.  The entry is allocated
-   in POOL or in the same pool as PARENT; the caller should copy if it
-   cares.  */
+   entry, set *ID_P to NULL but do not error.  The node-id is
+   allocated in POOL. */
 static svn_error_t *
-dir_entry_id_from_node(const svn_fs_id_t **id_p, 
+dir_entry_id_from_node(const svn_fs_id_t **id_p,
                        dag_node_t *parent,
                        const char *name,
                        apr_pool_t *pool)
 {
   apr_hash_t *entries;
   svn_fs_dirent_t *dirent;
+  apr_pool_t *subpool = svn_pool_create(pool);
 
-  SVN_ERR(svn_fs_fs__dag_dir_entries(&entries, parent, pool));
+  SVN_ERR(svn_fs_fs__dag_dir_entries(&entries, parent, subpool, pool));
   if (entries)
     dirent = apr_hash_get(entries, name, APR_HASH_KEY_STRING);
   else
     dirent = NULL;
-    
-  *id_p = dirent ? dirent->id : NULL;
+
+  *id_p = dirent ? svn_fs_fs__id_copy(dirent->id, pool) : NULL;
+
+  svn_pool_destroy(subpool);
+
   return SVN_NO_ERROR;
 }
 
@@ -299,7 +341,7 @@ set_entry(dag_node_t *parent,
   /* Set the new entry. */
   SVN_ERR(svn_fs_fs__set_entry(parent->fs, txn_id, parent_noderev, name, id,
                                kind, pool));
-  
+
   return SVN_NO_ERROR;
 }
 
@@ -333,7 +375,7 @@ make_entry(dag_node_t **child_p,
        _("Attempted to create entry in non-directory parent"));
 
   /* Check that the parent is mutable. */
-  if (! svn_fs_fs__dag_check_mutable(parent, txn_id))
+  if (! svn_fs_fs__dag_check_mutable(parent))
     return svn_error_createf
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
        _("Attempted to clone child of non-mutable node"));
@@ -349,7 +391,7 @@ make_entry(dag_node_t **child_p,
   new_noderev.copyroot_rev = parent_noderev->copyroot_rev;
   new_noderev.copyfrom_rev = SVN_INVALID_REVNUM;
   new_noderev.copyfrom_path = NULL;
-  
+
   SVN_ERR(svn_fs_fs__create_node
           (&new_node_id, svn_fs_fs__dag_get_fs(parent), &new_noderev,
            svn_fs_fs__id_copy_id(svn_fs_fs__dag_get_id(parent)),
@@ -372,11 +414,12 @@ make_entry(dag_node_t **child_p,
 svn_error_t *
 svn_fs_fs__dag_dir_entries(apr_hash_t **entries,
                            dag_node_t *node,
-                           apr_pool_t *pool)
+                           apr_pool_t *pool,
+                           apr_pool_t *node_pool)
 {
   node_revision_t *noderev;
 
-  SVN_ERR(get_node_revision(&noderev, node, pool));
+  SVN_ERR(get_node_revision(&noderev, node, node_pool));
 
   if (noderev->kind != svn_node_dir)
     return svn_error_create(SVN_ERR_FS_NOT_DIRECTORY, NULL,
@@ -399,9 +442,9 @@ svn_fs_fs__dag_set_entry(dag_node_t *node,
     return svn_error_create
       (SVN_ERR_FS_NOT_DIRECTORY, NULL,
        _("Attempted to set entry in non-directory node"));
-  
+
   /* Check it's mutable. */
-  if (! svn_fs_fs__dag_check_mutable(node, txn_id))
+  if (! svn_fs_fs__dag_check_mutable(node))
     return svn_error_create
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
        _("Attempted to set entry in immutable node"));
@@ -427,7 +470,7 @@ svn_fs_fs__dag_get_proplist(apr_hash_t **proplist_p,
                                   noderev, pool));
 
   *proplist_p = proplist;
-  
+
   return SVN_NO_ERROR;
 }
 
@@ -435,15 +478,14 @@ svn_fs_fs__dag_get_proplist(apr_hash_t **proplist_p,
 svn_error_t *
 svn_fs_fs__dag_set_proplist(dag_node_t *node,
                             apr_hash_t *proplist,
-                            const char *txn_id,
                             apr_pool_t *pool)
 {
   node_revision_t *noderev;
 
   /* Sanity check: this node better be mutable! */
-  if (! svn_fs_fs__dag_check_mutable(node, txn_id))
+  if (! svn_fs_fs__dag_check_mutable(node))
     {
-      svn_string_t *idstr = svn_fs_fs__id_unparse(node->id, node->pool);
+      svn_string_t *idstr = svn_fs_fs__id_unparse(node->id, pool);
       return svn_error_createf
         (SVN_ERR_FS_NOT_MUTABLE, NULL,
          "Can't set proplist on *immutable* node-revision %s",
@@ -460,6 +502,89 @@ svn_fs_fs__dag_set_proplist(dag_node_t *node,
 }
 
 
+svn_error_t *
+svn_fs_fs__dag_increment_mergeinfo_count(dag_node_t *node,
+                                         apr_int64_t increment,
+                                         apr_pool_t *pool)
+{
+  node_revision_t *noderev;
+
+  /* Sanity check: this node better be mutable! */
+  if (! svn_fs_fs__dag_check_mutable(node))
+    {
+      svn_string_t *idstr = svn_fs_fs__id_unparse(node->id, pool);
+      return svn_error_createf
+        (SVN_ERR_FS_NOT_MUTABLE, NULL,
+         "Can't increment mergeinfo count on *immutable* node-revision %s",
+         idstr->data);
+    }
+
+  if (increment == 0)
+    return SVN_NO_ERROR;
+
+  /* Go get a fresh NODE-REVISION for this node. */
+  SVN_ERR(get_node_revision(&noderev, node, pool));
+
+  noderev->mergeinfo_count += increment;
+  if (noderev->mergeinfo_count < 0)
+    {
+      svn_string_t *idstr = svn_fs_fs__id_unparse(node->id, pool);
+      return svn_error_createf
+        (SVN_ERR_FS_CORRUPT, NULL,
+         apr_psprintf(pool,
+                      _("Can't increment mergeinfo count on node-revision %%s "
+                        "to negative value %%%s"),
+                      APR_INT64_T_FMT),
+         idstr->data, noderev->mergeinfo_count);
+    }
+  if (noderev->mergeinfo_count > 1 && noderev->kind == svn_node_file)
+    {
+      svn_string_t *idstr = svn_fs_fs__id_unparse(node->id, pool);
+      return svn_error_createf
+        (SVN_ERR_FS_CORRUPT, NULL,
+         apr_psprintf(pool,
+                      _("Can't increment mergeinfo count on *file* "
+                        "node-revision %%s to %%%s (> 1)"),
+                      APR_INT64_T_FMT),
+         idstr->data, noderev->mergeinfo_count);
+    }
+
+  /* Flush it out. */
+  SVN_ERR(svn_fs_fs__put_node_revision(node->fs, noderev->id,
+                                       noderev, FALSE, pool));
+
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_fs_fs__dag_set_has_mergeinfo(dag_node_t *node,
+                                 svn_boolean_t has_mergeinfo,
+                                 apr_pool_t *pool)
+{
+  node_revision_t *noderev;
+
+  /* Sanity check: this node better be mutable! */
+  if (! svn_fs_fs__dag_check_mutable(node))
+    {
+      svn_string_t *idstr = svn_fs_fs__id_unparse(node->id, pool);
+      return svn_error_createf
+        (SVN_ERR_FS_NOT_MUTABLE, NULL,
+         "Can't set mergeinfo flag on *immutable* node-revision %s",
+         idstr->data);
+    }
+
+  /* Go get a fresh NODE-REVISION for this node. */
+  SVN_ERR(get_node_revision(&noderev, node, pool));
+
+  noderev->has_mergeinfo = has_mergeinfo;
+
+  /* Flush it out. */
+  SVN_ERR(svn_fs_fs__put_node_revision(node->fs, noderev->id,
+                                       noderev, FALSE, pool));
+
+  return SVN_NO_ERROR;
+}
+
 
 /*** Roots. ***/
 
@@ -483,7 +608,7 @@ svn_fs_fs__dag_txn_root(dag_node_t **node_p,
                         apr_pool_t *pool)
 {
   const svn_fs_id_t *root_id, *ignored;
-  
+
   SVN_ERR(svn_fs_fs__get_txn_ids(&root_id, &ignored, fs, txn_id, pool));
   return svn_fs_fs__dag_get_node(node_p, fs, root_id, pool);
 }
@@ -496,7 +621,7 @@ svn_fs_fs__dag_txn_base_root(dag_node_t **node_p,
                              apr_pool_t *pool)
 {
   const svn_fs_id_t *base_root_id, *ignored;
-  
+
   SVN_ERR(svn_fs_fs__get_txn_ids(&ignored, &base_root_id, fs, txn_id, pool));
   return svn_fs_fs__dag_get_node(node_p, fs, base_root_id, pool);
 }
@@ -517,14 +642,14 @@ svn_fs_fs__dag_clone_child(dag_node_t **child_p,
   svn_fs_t *fs = svn_fs_fs__dag_get_fs(parent);
 
   /* First check that the parent is mutable. */
-  if (! svn_fs_fs__dag_check_mutable(parent, txn_id))
-    return svn_error_createf 
+  if (! svn_fs_fs__dag_check_mutable(parent))
+    return svn_error_createf
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
        "Attempted to clone child of non-mutable node");
 
   /* Make sure that NAME is a single path component. */
   if (! svn_path_is_single_path_component(name))
-    return svn_error_createf 
+    return svn_error_createf
       (SVN_ERR_FS_NOT_SINGLE_PATH_COMPONENT, NULL,
        "Attempted to make a child clone with an illegal name '%s'", name);
 
@@ -533,7 +658,7 @@ svn_fs_fs__dag_clone_child(dag_node_t **child_p,
 
   /* Check for mutability in the node we found.  If it's mutable, we
      don't need to clone it. */
-  if (svn_fs_fs__dag_check_mutable(cur_entry, txn_id))
+  if (svn_fs_fs__dag_check_mutable(cur_entry))
     {
       /* This has already been cloned */
       new_node_id = cur_entry->id;
@@ -541,10 +666,10 @@ svn_fs_fs__dag_clone_child(dag_node_t **child_p,
   else
     {
       node_revision_t *noderev, *parent_noderev;
-      
+
       /* Go get a fresh NODE-REVISION for current child node. */
       SVN_ERR(get_node_revision(&noderev, cur_entry, pool));
-      
+
       if (is_parent_copyroot)
         {
           SVN_ERR(get_node_revision(&parent_noderev, parent, pool));
@@ -552,16 +677,16 @@ svn_fs_fs__dag_clone_child(dag_node_t **child_p,
           noderev->copyroot_path = apr_pstrdup(pool,
                                                parent_noderev->copyroot_path);
         }
-      
+
       noderev->copyfrom_path = NULL;
       noderev->copyfrom_rev = SVN_INVALID_REVNUM;
-      
+
       noderev->predecessor_id = svn_fs_fs__id_copy(cur_entry->id, pool);
       if (noderev->predecessor_count != -1)
         noderev->predecessor_count++;
       noderev->created_path = svn_path_join(parent_path, name, pool);
-      
-      SVN_ERR(svn_fs_fs__create_successor(&new_node_id, fs, cur_entry->id, 
+
+      SVN_ERR(svn_fs_fs__create_successor(&new_node_id, fs, cur_entry->id,
                                           noderev, copy_id, txn_id, pool));
 
       /* Replace the ID in the parent's ENTRY list with the ID which
@@ -583,7 +708,7 @@ svn_fs_fs__dag_clone_root(dag_node_t **root_p,
                           apr_pool_t *pool)
 {
   const svn_fs_id_t *base_root_id, *root_id;
-  
+
   /* Get the node ID's of the root directories of the transaction and
      its base revision.  */
   SVN_ERR(svn_fs_fs__get_txn_ids(&root_id, &base_root_id, fs, txn_id, pool));
@@ -591,7 +716,7 @@ svn_fs_fs__dag_clone_root(dag_node_t **root_p,
   /* Oh, give me a clone...
      (If they're the same, we haven't cloned the transaction's root
      directory yet.)  */
-  if (svn_fs_fs__id_eq(root_id, base_root_id)) 
+  if (svn_fs_fs__id_eq(root_id, base_root_id))
     {
       abort();
     }
@@ -608,13 +733,6 @@ svn_fs_fs__dag_clone_root(dag_node_t **root_p,
 }
 
 
-/* Delete the directory entry named NAME from PARENT, allocating from
-   POOL.  PARENT must be mutable.  NAME must be a single path
-   component.  If REQUIRE_EMPTY is true and the node being deleted is
-   a directory, it must be empty.
-
-   If return SVN_ERR_FS_NO_SUCH_ENTRY, then there is no entry NAME in
-   PARENT.  */
 svn_error_t *
 svn_fs_fs__dag_delete(dag_node_t *parent,
                       const char *name,
@@ -626,7 +744,7 @@ svn_fs_fs__dag_delete(dag_node_t *parent,
   svn_fs_t *fs = parent->fs;
   svn_fs_dirent_t *dirent;
   svn_fs_id_t *id;
-  dag_node_t *node;
+  apr_pool_t *subpool;
 
   /* Make sure parent is a directory. */
   if (parent->kind != svn_node_dir)
@@ -635,7 +753,7 @@ svn_fs_fs__dag_delete(dag_node_t *parent,
        "Attempted to delete entry '%s' from *non*-directory node", name);
 
   /* Make sure parent is mutable. */
-  if (! svn_fs_fs__dag_check_mutable(parent, txn_id))
+  if (! svn_fs_fs__dag_check_mutable(parent))
     return svn_error_createf
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
        "Attempted to delete entry '%s' from immutable directory node", name);
@@ -649,8 +767,10 @@ svn_fs_fs__dag_delete(dag_node_t *parent,
   /* Get a fresh NODE-REVISION for the parent node. */
   SVN_ERR(get_node_revision(&parent_noderev, parent, pool));
 
+  subpool = svn_pool_create(pool);
+
   /* Get a dirent hash for this directory. */
-  SVN_ERR(svn_fs_fs__rep_contents_dir(&entries, fs, parent_noderev, pool));
+  SVN_ERR(svn_fs_fs__rep_contents_dir(&entries, fs, parent_noderev, subpool));
 
   /* Find name in the ENTRIES hash. */
   dirent = apr_hash_get(entries, name, APR_HASH_KEY_STRING);
@@ -663,21 +783,18 @@ svn_fs_fs__dag_delete(dag_node_t *parent,
       (SVN_ERR_FS_NO_SUCH_ENTRY, NULL,
        "Delete failed--directory has no entry '%s'", name);
 
-  /* Stash a copy of the ID, since dirent will become invalid during
-     svn_fs_fs__dag_delete_if_mutable. */
+  /* Copy the ID out of the subpool and release the rest of the
+     directory listing. */
   id = svn_fs_fs__id_copy(dirent->id, pool);
-  
-  /* Use the ID to get the entry's node.  */
-  SVN_ERR(svn_fs_fs__dag_get_node(&node, svn_fs_fs__dag_get_fs(parent), id,
-                                  pool));
+  svn_pool_destroy(subpool);
 
   /* If mutable, remove it and any mutable children from db. */
-  SVN_ERR(svn_fs_fs__dag_delete_if_mutable(parent->fs, id, txn_id, pool));
+  SVN_ERR(svn_fs_fs__dag_delete_if_mutable(parent->fs, id, pool));
 
   /* Remove this entry from its parent's entries list. */
   SVN_ERR(svn_fs_fs__set_entry(parent->fs, txn_id, parent_noderev, name,
                                NULL, svn_node_unknown, pool));
-    
+
   return SVN_NO_ERROR;
 }
 
@@ -685,7 +802,6 @@ svn_fs_fs__dag_delete(dag_node_t *parent,
 svn_error_t *
 svn_fs_fs__dag_remove_node(svn_fs_t *fs,
                            const svn_fs_id_t *id,
-                           const char *txn_id,
                            apr_pool_t *pool)
 {
   dag_node_t *node;
@@ -694,7 +810,7 @@ svn_fs_fs__dag_remove_node(svn_fs_t *fs,
   SVN_ERR(svn_fs_fs__dag_get_node(&node, fs, id, pool));
 
   /* If immutable, do nothing and return immediately. */
-  if (! svn_fs_fs__dag_check_mutable(node, txn_id))
+  if (! svn_fs_fs__dag_check_mutable(node))
     return svn_error_createf(SVN_ERR_FS_NOT_MUTABLE, NULL,
                              "Attempted removal of immutable node");
 
@@ -708,7 +824,6 @@ svn_fs_fs__dag_remove_node(svn_fs_t *fs,
 svn_error_t *
 svn_fs_fs__dag_delete_if_mutable(svn_fs_t *fs,
                                  const svn_fs_id_t *id,
-                                 const char *txn_id,
                                  apr_pool_t *pool)
 {
   dag_node_t *node;
@@ -717,7 +832,7 @@ svn_fs_fs__dag_delete_if_mutable(svn_fs_t *fs,
   SVN_ERR(svn_fs_fs__dag_get_node(&node, fs, id, pool));
 
   /* If immutable, do nothing and return immediately. */
-  if (! svn_fs_fs__dag_check_mutable(node, txn_id))
+  if (! svn_fs_fs__dag_check_mutable(node))
     return SVN_NO_ERROR;
 
   /* Else it's mutable.  Recurse on directories... */
@@ -727,8 +842,7 @@ svn_fs_fs__dag_delete_if_mutable(svn_fs_t *fs,
       apr_hash_index_t *hi;
 
       /* Loop over hash entries */
-      SVN_ERR(svn_fs_fs__dag_dir_entries(&entries, node, pool));
-      entries = svn_fs_fs__copy_dir_entries(entries, pool);
+      SVN_ERR(svn_fs_fs__dag_dir_entries(&entries, node, pool, pool));
       if (entries)
         {
           for (hi = apr_hash_first(pool, entries);
@@ -741,14 +855,14 @@ svn_fs_fs__dag_delete_if_mutable(svn_fs_t *fs,
               apr_hash_this(hi, NULL, NULL, &val);
               dirent = val;
               SVN_ERR(svn_fs_fs__dag_delete_if_mutable(fs, dirent->id,
-                                                       txn_id, pool));
+                                                       pool));
             }
         }
     }
 
   /* ... then delete the node itself, after deleting any mutable
      representations and strings it points to. */
-  SVN_ERR(svn_fs_fs__dag_remove_node(fs, id, txn_id, pool));
+  SVN_ERR(svn_fs_fs__dag_remove_node(fs, id, pool));
 
   return SVN_NO_ERROR;
 }
@@ -758,7 +872,7 @@ svn_fs_fs__dag_make_file(dag_node_t **child_p,
                          dag_node_t *parent,
                          const char *parent_path,
                          const char *name,
-                         const char *txn_id, 
+                         const char *txn_id,
                          apr_pool_t *pool)
 {
   /* Call our little helper function */
@@ -771,7 +885,7 @@ svn_fs_fs__dag_make_dir(dag_node_t **child_p,
                         dag_node_t *parent,
                         const char *parent_path,
                         const char *name,
-                        const char *txn_id, 
+                        const char *txn_id,
                         apr_pool_t *pool)
 {
   /* Call our little helper function */
@@ -783,16 +897,16 @@ svn_error_t *
 svn_fs_fs__dag_get_contents(svn_stream_t **contents_p,
                             dag_node_t *file,
                             apr_pool_t *pool)
-{ 
+{
   node_revision_t *noderev;
   svn_stream_t *contents;
 
   /* Make sure our node is a file. */
   if (file->kind != svn_node_file)
-    return svn_error_createf 
+    return svn_error_createf
       (SVN_ERR_FS_NOT_FILE, NULL,
        "Attempted to get textual contents of a *non*-file node");
-  
+
   /* Go get a fresh node-revision for FILE. */
   SVN_ERR(get_node_revision(&noderev, file, pool));
 
@@ -818,10 +932,10 @@ svn_fs_fs__dag_get_file_delta_stream(svn_txdelta_stream_t **stream_p,
   /* Make sure our nodes are files. */
   if ((source && source->kind != svn_node_file)
       || target->kind != svn_node_file)
-    return svn_error_createf 
+    return svn_error_createf
       (SVN_ERR_FS_NOT_FILE, NULL,
        "Attempted to get textual contents of a *non*-file node");
-  
+
   /* Go get fresh node-revisions for the nodes. */
   if (source)
     SVN_ERR(get_node_revision(&src_noderev, source, pool));
@@ -841,12 +955,12 @@ svn_error_t *
 svn_fs_fs__dag_file_length(svn_filesize_t *length,
                            dag_node_t *file,
                            apr_pool_t *pool)
-{ 
+{
   node_revision_t *noderev;
 
   /* Make sure our node is a file. */
   if (file->kind != svn_node_file)
-    return svn_error_createf 
+    return svn_error_createf
       (SVN_ERR_FS_NOT_FILE, NULL,
        "Attempted to get length of a *non*-file node");
 
@@ -863,11 +977,11 @@ svn_error_t *
 svn_fs_fs__dag_file_checksum(unsigned char digest[],
                              dag_node_t *file,
                              apr_pool_t *pool)
-{ 
+{
   node_revision_t *noderev;
 
   if (file->kind != svn_node_file)
-    return svn_error_createf 
+    return svn_error_createf
       (SVN_ERR_FS_NOT_FILE, NULL,
        "Attempted to get checksum of a *non*-file node");
 
@@ -882,7 +996,6 @@ svn_fs_fs__dag_file_checksum(unsigned char digest[],
 svn_error_t *
 svn_fs_fs__dag_get_edit_stream(svn_stream_t **contents,
                                dag_node_t *file,
-                               const char *txn_id,
                                apr_pool_t *pool)
 {
   node_revision_t *noderev;
@@ -890,13 +1003,13 @@ svn_fs_fs__dag_get_edit_stream(svn_stream_t **contents,
 
   /* Make sure our node is a file. */
   if (file->kind != svn_node_file)
-    return svn_error_createf 
+    return svn_error_createf
       (SVN_ERR_FS_NOT_FILE, NULL,
        "Attempted to set textual contents of a *non*-file node");
-  
+
   /* Make sure our node is mutable. */
-  if (! svn_fs_fs__dag_check_mutable(file, txn_id))
-    return svn_error_createf 
+  if (! svn_fs_fs__dag_check_mutable(file))
+    return svn_error_createf
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
        "Attempted to set textual contents of an immutable node");
 
@@ -906,7 +1019,7 @@ svn_fs_fs__dag_get_edit_stream(svn_stream_t **contents,
   SVN_ERR(svn_fs_fs__set_contents(&ws, file->fs, noderev, pool));
 
   *contents = ws;
-  
+
   return SVN_NO_ERROR;
 }
 
@@ -915,7 +1028,6 @@ svn_fs_fs__dag_get_edit_stream(svn_stream_t **contents,
 svn_error_t *
 svn_fs_fs__dag_finalize_edits(dag_node_t *file,
                               const char *checksum,
-                              const char *txn_id, 
                               apr_pool_t *pool)
 {
   unsigned char digest[APR_MD5_DIGESTSIZE];
@@ -945,14 +1057,19 @@ svn_fs_fs__dag_dup(dag_node_t *node,
   dag_node_t *new_node = apr_pcalloc(pool, sizeof(*new_node));
 
   new_node->fs = node->fs;
-  new_node->pool = pool;
   new_node->id = svn_fs_fs__id_copy(node->id, pool);
   new_node->kind = node->kind;
   new_node->created_path = apr_pstrdup(pool, node->created_path);
 
-  /* Leave new_node->node_revision zero for now, so it'll get read in.
-     We can get fancy and duplicate node's cache later.  */
-
+  /* Only copy cached node_revision_t for immutable nodes. */
+  if (node->node_revision && !svn_fs_fs__dag_check_mutable(node))
+    {
+      new_node->node_revision = copy_node_revision(node->node_revision, pool);
+      new_node->node_revision->id =
+          svn_fs_fs__id_copy(node->node_revision->id, pool);
+      new_node->node_revision->is_fresh_txn_root =
+          node->node_revision->is_fresh_txn_root;
+    }
   return new_node;
 }
 
@@ -968,13 +1085,13 @@ svn_fs_fs__dag_open(dag_node_t **child_p,
   /* Ensure that NAME exists in PARENT's entry list. */
   SVN_ERR(dir_entry_id_from_node(&node_id, parent, name, pool));
   if (! node_id)
-    return svn_error_createf 
+    return svn_error_createf
       (SVN_ERR_FS_NOT_FOUND, NULL,
        "Attempted to open non-existent child node '%s'", name);
-  
+
   /* Make sure that NAME is a single path component. */
   if (! svn_path_is_single_path_component(name))
-    return svn_error_createf 
+    return svn_error_createf
       (SVN_ERR_FS_NOT_SINGLE_PATH_COMPONENT, NULL,
        "Attempted to open node with an illegal name '%s'", name);
 
@@ -991,18 +1108,18 @@ svn_fs_fs__dag_copy(dag_node_t *to_node,
                     svn_boolean_t preserve_history,
                     svn_revnum_t from_rev,
                     const char *from_path,
-                    const char *txn_id, 
+                    const char *txn_id,
                     apr_pool_t *pool)
 {
   const svn_fs_id_t *id;
-  
+
   if (preserve_history)
     {
       node_revision_t *from_noderev, *to_noderev;
       const char *copy_id;
       const svn_fs_id_t *src_id = svn_fs_fs__dag_get_id(from_node);
       svn_fs_t *fs = svn_fs_fs__dag_get_fs(from_node);
-      
+
       /* Make a copy of the original node revision. */
       SVN_ERR(get_node_revision(&from_noderev, from_node, pool));
       to_noderev = copy_node_revision(from_noderev, pool);
@@ -1032,7 +1149,7 @@ svn_fs_fs__dag_copy(dag_node_t *to_node,
     {
       id = svn_fs_fs__dag_get_id(from_node);
     }
-      
+
   /* Set the entry in to_node to the new id. */
   SVN_ERR(svn_fs_fs__dag_set_entry(to_node, entry, id, from_node->kind,
                                    txn_id, pool));
@@ -1045,11 +1162,11 @@ svn_fs_fs__dag_copy(dag_node_t *to_node,
 /*** Comparison. ***/
 
 svn_error_t *
-svn_fs_fs__things_different(svn_boolean_t *props_changed,
-                            svn_boolean_t *contents_changed,
-                            dag_node_t *node1,
-                            dag_node_t *node2,
-                            apr_pool_t *pool)
+svn_fs_fs__dag_things_different(svn_boolean_t *props_changed,
+                                svn_boolean_t *contents_changed,
+                                dag_node_t *node1,
+                                dag_node_t *node2,
+                                apr_pool_t *pool)
 {
   node_revision_t *noderev1, *noderev2;
 
@@ -1069,10 +1186,10 @@ svn_fs_fs__things_different(svn_boolean_t *props_changed,
 
   /* Compare contents keys. */
   if (contents_changed != NULL)
-    *contents_changed = 
+    *contents_changed =
       (! svn_fs_fs__noderev_same_rep_key(noderev1->data_rep,
                                          noderev2->data_rep));
-  
+
   return SVN_NO_ERROR;
 }
 
@@ -1083,7 +1200,7 @@ svn_fs_fs__dag_get_copyroot(svn_revnum_t *rev,
                             apr_pool_t *pool)
 {
   node_revision_t *noderev;
-  
+
   /* Go get a fresh node-revision for FILE. */
   SVN_ERR(get_node_revision(&noderev, node, pool));
 
@@ -1099,7 +1216,7 @@ svn_fs_fs__dag_get_copyfrom_rev(svn_revnum_t *rev,
                                 apr_pool_t *pool)
 {
   node_revision_t *noderev;
-  
+
   /* Go get a fresh node-revision for FILE. */
   SVN_ERR(get_node_revision(&noderev, node, pool));
 
@@ -1114,11 +1231,11 @@ svn_fs_fs__dag_get_copyfrom_path(const char **path,
                                  apr_pool_t *pool)
 {
   node_revision_t *noderev;
-  
+
   /* Go get a fresh node-revision for FILE. */
   SVN_ERR(get_node_revision(&noderev, node, pool));
 
   *path = noderev->copyfrom_path;
-  
+
   return SVN_NO_ERROR;
 }
