@@ -1,7 +1,7 @@
 /* fs.h : interface to Subversion filesystem, private to libsvn_fs
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -22,7 +22,10 @@
 #include <apr_hash.h>
 #include <apr_md5.h>
 #include <apr_thread_mutex.h>
+#include <apr_network_io.h>
+
 #include "svn_fs.h"
+#include "private/svn_fs_private.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -31,20 +34,71 @@ extern "C" {
 
 /*** The filesystem structure.  ***/
 
+/* Following are defines that specify the textual elements of the
+   native filesystem directories and revision files. */
+
+/* Names of special files in the fs_fs filesystem. */
+#define PATH_FORMAT           "format"           /* Contains format number */
+#define PATH_UUID             "uuid"             /* Contains UUID */
+#define PATH_CURRENT          "current"          /* Youngest revision */
+#define PATH_LOCK_FILE        "write-lock"       /* Revision lock file */
+#define PATH_REVS_DIR         "revs"             /* Directory of revisions */
+#define PATH_REVPROPS_DIR     "revprops"         /* Directory of revprops */
+#define PATH_TXNS_DIR         "transactions"     /* Directory of transactions */
+#define PATH_NODE_ORIGINS_DIR "node-origins"     /* Lazy node-origin cache */
+#define PATH_TXN_PROTOS_DIR   "txn-protorevs"    /* Directory of proto-revs */
+#define PATH_TXN_CURRENT      "txn-current"      /* File with next txn key */
+#define PATH_TXN_CURRENT_LOCK "txn-current-lock" /* Lock for txn-current */
+#define PATH_LOCKS_DIR        "locks"            /* Directory of locks */
+
+/* Names of special files and file extensions for transactions */
+#define PATH_CHANGES       "changes"       /* Records changes made so far */
+#define PATH_TXN_PROPS     "props"         /* Transaction properties */
+#define PATH_NEXT_IDS      "next-ids"      /* Next temporary ID assignments */
+#define PATH_PREFIX_NODE   "node."         /* Prefix for node filename */
+#define PATH_EXT_TXN       ".txn"          /* Extension of txn dir */
+#define PATH_EXT_CHILDREN  ".children"     /* Extension for dir contents */
+#define PATH_EXT_PROPS     ".props"        /* Extension for node props */
+#define PATH_EXT_REV       ".rev"          /* Extension of protorev file */
+#define PATH_EXT_REV_LOCK  ".rev-lock"     /* Extension of protorev lock file */
+/* Names of files in legacy FS formats */
+#define PATH_REV           "rev"           /* Proto rev file */
+#define PATH_REV_LOCK      "rev-lock"      /* Proto rev (write) lock file */
+
 /* The format number of this filesystem.
    This is independent of the repository format number, and
    independent of any other FS back ends. */
-#define SVN_FS_FS__FORMAT_NUMBER   2
+#define SVN_FS_FS__FORMAT_NUMBER   3
 
 /* The minimum format number that supports svndiff version 1.  */
 #define SVN_FS_FS__MIN_SVNDIFF1_FORMAT 2
 
-/* Maximum number of directories to cache dirents for. 
-   This *must* be a power of 2 for DIR_CACHE_ENTRIES_INDEX
+/* The minimum format number that supports transaction ID generation
+   using a transaction sequence in the txn-current file. */
+#define SVN_FS_FS__MIN_TXN_CURRENT_FORMAT 3
+
+/* The minimum format number that supports the "layout" filesystem
+   format option. */
+#define SVN_FS_FS__MIN_LAYOUT_FORMAT_OPTION_FORMAT 3
+
+/* The minimum format number that stores protorevs in a separate directory. */
+#define SVN_FS_FS__MIN_PROTOREVS_DIR_FORMAT 3
+
+/* The minimum format number that doesn't keep node and copy ID counters. */
+#define SVN_FS_FS__MIN_NO_GLOBAL_IDS_FORMAT 3
+
+/* The minimum format number that maintains minfo-here and minfo-count
+   noderev fields. */
+#define SVN_FS_FS__MIN_MERGEINFO_FORMAT 3
+
+/* Maximum number of directories to cache dirents for.
+   This *must* be a power of 2 for DIR_CACHE_ENTRIES_MASK
    to work.  */
 #define NUM_DIR_CACHE_ENTRIES 128
 #define DIR_CACHE_ENTRIES_MASK(x) ((x) & (NUM_DIR_CACHE_ENTRIES - 1))
 
+/* Maximum number of revroot ids to cache dirents for at a time. */
+#define NUM_RRI_CACHE_ENTRIES 4096
 
 /* Private FSFS-specific data shared between all svn_txn_t objects that
    relate to a particular transaction in a filesystem (as identified
@@ -57,10 +111,13 @@ typedef struct fs_fs_shared_txn_data_t
      transaction. */
   struct fs_fs_shared_txn_data_t *next;
 
-  /* This transaction's ID.  This is in the form <rev>-<uniqueifier>,
-     where <uniqueifier> runs from 0-99999 (see create_txn_dir() in
-     fs_fs.c). */
-  char txn_id[18];
+  /* This transaction's ID.  For repositories whose format is less
+     than SVN_FS_FS__MIN_TXN_CURRENT_FORMAT, the ID is in the form
+     <rev>-<uniqueifier>, where <uniqueifier> runs from 0-99999 (see
+     create_txn_dir_pre_1_5() in fs_fs.c).  For newer repositories,
+     the form is <rev>-<200 digit base 36 number> (see
+     create_txn_dir() in fs_fs.c). */
+  char txn_id[SVN_FS__TXN_MAX_LEN+1];
 
   /* Whether the transaction's prototype revision file is locked for
      writing by any thread in this process (including the current
@@ -96,6 +153,10 @@ typedef struct
   /* A lock for intra-process synchronization when grabbing the
      repository write lock. */
   apr_thread_mutex_t *fs_write_lock;
+
+  /* A lock for intra-process synchronization when locking the
+     txn-current file. */
+  apr_thread_mutex_t *txn_current_lock;
 #endif
 
   /* The common pool, under which this object is allocated, subpools
@@ -103,6 +164,24 @@ typedef struct
   apr_pool_t *common_pool;
 } fs_fs_shared_data_t;
 
+typedef struct dag_node_t dag_node_t;
+
+/* Structure for DAG-node cache.  Cache items are arranged in a
+   circular LRU list with a dummy entry, and also indexed with a hash
+   table.  Transaction nodes are cached within the individual txn
+   roots; revision nodes are cached together within the FS object. */
+typedef struct dag_node_cache_t
+{
+  const char *key;                /* Lookup key for cached node: path
+                                     for txns; rev catenated with path
+                                     for revs */
+  dag_node_t *node;               /* Cached node */
+  struct dag_node_cache_t *prev;  /* Next node in LRU list */
+  struct dag_node_cache_t *next;  /* Previous node in LRU list */
+  apr_pool_t *pool;               /* Pool in which node is allocated */
+} dag_node_cache_t;
+
+
 /* Private (non-shared) FSFS-specific data for each svn_fs_t object. */
 typedef struct
 {
@@ -113,43 +192,42 @@ typedef struct
 
   /* The format number of this FS. */
   int format;
+  /* The maximum number of files to store per directory (for sharded
+     layouts) or zero (for linear layouts). */
+  int max_files_per_dir;
 
   /* The uuid of this FS. */
   const char *uuid;
 
+  /* The revision that was youngest, last time we checked. */
+  svn_revnum_t youngest_rev_cache;
+
+  /* Caches of immutable data.
+     
+     Both of these could be moved to fs_fs_shared_data_t to make them
+     last longer; on the other hand, this would require adding mutexes
+     for threaded builds.
+  */
+
+  /* A cache of revision root IDs, allocated in this subpool.  (IDs
+     are so small that one pool per ID would be overkill;
+     unfortunately, this means the only way we expire cache entries is
+     by wiping the whole cache.) */
+  apr_hash_t *rev_root_id_cache;
+  apr_pool_t *rev_root_id_cache_pool;
+
+  /* DAG node cache for immutable nodes */
+  dag_node_cache_t rev_node_list;
+  apr_hash_t *rev_node_cache;
+
   /* Data shared between all svn_fs_t objects for a given filesystem. */
   fs_fs_shared_data_t *shared;
 } fs_fs_data_t;
 
 
-/* Return a canonicalized version of a filesystem PATH, allocated in
-   POOL.  While the filesystem API is pretty flexible about the
-   incoming paths (they must be UTF-8 with '/' as separators, but they
-   don't have to begin with '/', and multiple contiguous '/'s are
-   ignored) we want any paths that are physically stored in the
-   underlying database to look consistent.  Specifically, absolute
-   filesystem paths should begin with '/', and all redundant and trailing '/'
-   characters be removed.  */
-const char *
-svn_fs_fs__canonicalize_abspath(const char *path, apr_pool_t *pool);
-
-
-/*** Transaction Kind ***/
-typedef enum
-{
-  transaction_kind_normal = 1,  /* normal, uncommitted */
-  transaction_kind_committed,   /* committed */
-  transaction_kind_dead         /* uncommitted and dead */
-
-} transaction_kind_t;
-
-
 /*** Filesystem Transaction ***/
 typedef struct
 {
-  /* kind of transaction. */
-  transaction_kind_t kind;
-
   /* property list (const char * name, svn_string_t * value).
      may be NULL if there are no properties.  */
   apr_hash_t *proplist;
@@ -169,12 +247,14 @@ typedef struct
 
 
 /*** Representation ***/
+/* If you add fields to this, check to see if you need to change
+ * svn_fs_fs__rep_copy. */
 typedef struct
 {
   /* MD5 checksum for the contents produced by this representation.
      This checksum is for the contents the rep shows to consumers,
      regardless of how the rep stores the data under the hood.  It is
-     independent of the storage (fulltext, delta, whatever). 
+     independent of the storage (fulltext, delta, whatever).
 
      If all the bytes are 0, then for compatibility behave as though
      this checksum matches the expected checksum. */
@@ -200,6 +280,8 @@ typedef struct
 
 
 /*** Node-Revision ***/
+/* If you add fields to this, check to see if you need to change
+ * copy_node_revision in dag.c. */
 typedef struct
 {
   /* node kind */
@@ -220,7 +302,7 @@ typedef struct
      this node-rev was copied. */
   svn_revnum_t copyroot_rev;
   const char *copyroot_path;
-  
+
   /* number of predecessors this node revision has (recursively), or
      -1 if not known (for backward compatibility). */
   int predecessor_count;
@@ -232,13 +314,20 @@ typedef struct
   /* representation for this node's data.  may be NULL if there is
      no data. */
   representation_t *data_rep;
-  
+
   /* path at which this node first came into existence.  */
   const char *created_path;
 
   /* is this the unmodified root of a transaction? */
   svn_boolean_t is_fresh_txn_root;
 
+  /* Number of nodes with svn:mergeinfo properties that are
+     descendants of this node (including it itself) */
+  apr_int64_t mergeinfo_count;
+
+  /* Does this node itself have svn:mergeinfo? */
+  svn_boolean_t has_mergeinfo;
+
 } node_revision_t;
 
 
